@@ -7,10 +7,11 @@ use Net::DNS;
 use Getopt::Long qw(:config posix_default bundling);
 
 #### Options
-my $resolver;
+my ($resolver,$spfdfl);
 GetOptions(
     'd|debug' => sub { Mail::SPF::Iterator->import( Debug => 1 ) },
     'h|help' => sub { usage() },
+    'spfdfl=s' => \$spfdfl,
     'dns=s'  => sub {
 	my ($ip,$port) = $_[1] =~m{^([^:]+)(?::(\d+))?\z} or die $_[1];
 	$resolver = Net::DNS::Resolver->new( nameservers => [$ip]);
@@ -22,7 +23,8 @@ my ($ip,$sender,$helo,$local) = @ARGV;
 ($ip && $sender) or usage();
 
 #### SPF lookup
-my $spf = Mail::SPF::Iterator->new( $ip,$sender,$helo || '',$local );
+my $spf = Mail::SPF::Iterator->new($ip, $sender, $helo || '', $local,
+    { default_spf => $spfdfl });
 my $result = $spf->lookup_blocking(undef,$resolver);
 print "Received-SPF: ".$spf->mailheader."\n";
 print "Explanation: ".($spf->result)[3]."\n" if $result eq SPF_Fail;
@@ -31,7 +33,7 @@ print "Explanation: ".($spf->result)[3]."\n" if $result eq SPF_Fail;
 #### USAGE
 sub usage { die <<USAGE; }
 
- Usage: $0 [-d|--debug] [--dns dnsip[:port]] Ip Sender [Helo] [Localname]
+ Usage: $0 [-d|--debug] [--dns dnsip[:port]] [--spfdfl spf] Ip Sender [Helo] [Localname]
  lookup SPF result, returns SPF-Received header
  Example: 
  $0 10.0.3.4 user\@example.com smtp.example.com smtp.example.local
